C @(#)inteaper.for	19.1 (ESO-DMD) 02/25/03 13:19:46
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
C MA 02139, USA.
C
C Correspondence concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C.IDENTIFICATION: INTEAPER.FOR
C.AUTHOR:         Ch. Ounnas                  ESO - Garching
C.LANGUAGE:       F77+ESOext
C.KEYWORDS:       Image display, cursor, flux-magnitude
C.PURPOSE:        read the position of the two cursors on the DeAnza display
C                 and get integrated density of relevant pixels inside an 
C                 aperture
C.ALGORITHM:      Use enabled cursor(s) and read screen pixels, real pixels and
C                 world coordinatE when the ENTER button is pressed on the
C                 cursor board. Exit by pressing ENTER with enabled cursor(s) 
C                 off (see COORD.FOR from K. Banse).
C.INPUT/OUTPUT:   the following keywords are used:
C                 DEANZA/I/1/6   main DeAnza info
C                 DAZHOLD/I/1/3  cursor(s) enabled, cursor form(s) + split
C                                screen mode
C                 INPUTR/R/1/1                 radius of the aperture
C                 INPUTC/C/1/12                name of output table
C.VERSION:        830610  CO  creation
C 
C 021129		last modif
C 
C-------------------------------------------------------------------
 
      PROGRAM  INTAPR
C
      IMPLICIT     NONE
C
      DOUBLE PRECISION START(3),STEP(3)
      DOUBLE PRECISION CRDIN(3),CRDOUT(3)
C 
      INTEGER      MADRID
      INTEGER      IMF
      INTEGER      ST1,ST2,ISTAT,COORDC,COOFF
      INTEGER      INCURS,INFLAG,IAV
      INTEGER      NAXIS,I
      INTEGER      TIDI,TIDO,NROW,NCOL,NOP,N1,N2,ND1,ND2,NACOL,NAROW
      INTEGER      XY1(2),XY2(2)
      INTEGER      NPIX(3),ICOL(2),OCOL(7)
      INTEGER      XFIG(2048),YFIG(2048)
      INTEGER      COORD(3),NX,NY
      INTEGER      KUN,KNUL,PIXFLG,TABNUL(2)
      INTEGER*8    IP,WPTR
C
      CHARACTER    IDENT*72,CUNIT*64
      CHARACTER    DSCTYP*1,DSCNAM*20
      CHARACTER*60 FRAME,INTAB,OUTAB
      CHARACTER    CERCLE*2
      CHARACTER*16 COLX,COLY
      CHARACTER*16 UNITO(7),LABELO(7)
      CHARACTER*16 TABFOR(7)
      CHARACTER    TEXT*80

C
      REAL         RBUF(17), RINF(6), TEMP(2)
      REAL         PXLS1(6), PXLS2(6)
      REAL         XE,YE,RAYON,BEAM,BGSB,XEYE(2),ACAT(7)
      REAL         RX,RY
      REAL         XSTA,YSTA,XSTE,YSTE
C
      LOGICAL      SELFLG
C
      INCLUDE      'MID_INCLUDE:ST_DEF.INC'
      INCLUDE      'MID_INCLUDE:IDIDEV.INC'
      INCLUDE      'MID_INCLUDE:IDIMEM.INC'

      COMMON       /VMR/MADRID(1)

      INCLUDE      'MID_INCLUDE:ST_DAT.INC' 
C
      DATA         UNITO /'WORLD COORD','WORLD COORD','USER UNIT',' ',
     +                    ' ',' ',' '/
      DATA         LABELO/'X_COORD','Y_COORD','RADIUS ','NPIX ',
     +                    'FLUX ','BGSB ','MAG '/
      DATA         TABFOR/'G13.6','G13.6','G13.6','F6.0',
     +                    'G13.6','G13.6','G13.6'/
      DATA         DSCNAM/'BACKGROUND'/
      DATA         CERCLE/'CI'/
      DATA         IN_A/' '/, IN_B/' '/, OUTAB/' '/
C
902   FORMAT(G11.5,1X,G11.5,1X,G10.4,1X,I6,1X,G10.4,2X,
     +       G12.6,1X,G12.6)
C
C *** initialize MIDAS
      CALL STSPRO('INTAPR')
      COORDC = 100
C
C *** get input frame + map it
      CALL STKRDC('IN_A',1,1,60,IAV,FRAME,KUN,KNUL,ISTAT)
      IF (FRAME(1:1).EQ.'*') THEN                   !File loaded in display
         CALL CLNFRA(FRAME,FRAME,0)  
         INCURS = 1                                 !use cursor
      ELSE  
         INCURS = 0                           
      END IF

      IMF = -1
      CALL STIGET(FRAME,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,3,NAXIS,NPIX,
     +            START,STEP,IDENT,CUNIT,IP,IMF,ISTAT)
      XSTA = START(1)
      YSTA = START(2)
      XSTE = STEP(1)
      YSTE = STEP(2)
C
C *** get the subarea input options
      CALL STKRDC('IN_B',1,1,60,IAV,INTAB,KUN,KNUL,ISTAT)
      IF (INTAB(1:1).EQ.'*') THEN  
         IF (INCURS.EQ.0) THEN
            CALL STTPUT('Warning: displayed frame will be used,',ISTAT)
            CALL STTPUT('not the one given in the command line!',ISTAT)
         ENDIF
         INFLAG = 1                                 !INFLAG = 1 for cursor
      ELSE                                          !INFLAG = 2 for table
         INFLAG = 2
      END IF
C
C *** read INPUTC to fill OUTAB if output table
      CALL STKRDC('INPUTC',1,1,60,IAV,OUTAB,KUN,KNUL,ISTAT)
      IF (OUTAB(1:1).NE.'?') THEN                   !new table with 7 columns
         CALL TBTINI(OUTAB,0,F_O_MODE,7,COORDC,TIDO,ISTAT)
         DO 60, IAV = 1,7
            CALL TBCINI(TIDO,D_R4_FORMAT,1,TABFOR(IAV),UNITO(IAV),
     +                  LABELO(IAV),OCOL(IAV),ISTAT)
   60    CONTINUE
      END IF
C
C *** does the descriptor BACKGROUND exist in FRAME ??
      CALL STDFND(IMF,DSCNAM,DSCTYP,ND1,ND2,ISTAT)
      IF (DSCTYP.EQ.' ') THEN
         BGSB   = 0.
      ELSE
         CALL STDRDR(IMF,DSCNAM,1,1,IAV,BGSB,KUN,KNUL,ISTAT)
      END IF
C
C *** get the radius of the aperture and pixel_spec flag
      CALL STKRDR('INPUTR',1,1,IAV,BEAM,KUN,KNUL,ISTAT) 
      CALL STKRDC('P4',1,1,1,IAV,TEXT,KUN,KNUL,ISTAT) 
      IF ((TEXT(1:1).EQ.'F') .OR. (TEXT(1:1).EQ.'f')) THEN
         PIXFLG = 0                 ! use radius in frame pixels
      ELSE
         PIXFLG = 1                 ! use radius in world coord
      ENDIF
C  
      IF (INFLAG.EQ.1) THEN
         CALL STFXMP(2048,D_R4_FORMAT,WPTR,ISTAT)   ! map 2K pixel space
         CALL DTOPEN(1,ISTAT)                       ! attach display
C
C ***    get scroll values of displayed channel + scroll overlay accordingly
         CALL DTGICH(QDSPNO,QIMCH,FRAME,RINF,ISTAT)
         CALL DAZSCR(QDSPNO,QIMCH,SCROLX,SCROLY,ISTAT)
         COORDC = 0
         COOFF = 0
         FRAME  = ' '
C
C ***    read cursor position(s)
   10    CALL GETCUR('NNYY',FRAME,XY1,PXLS1(3),PXLS1(5),RBUF(1),ST1,
     +                            XY2,PXLS2(3),PXLS2(5),RBUF(1),ST2)
         IF ((ST1.EQ.0) .AND. (ST2.EQ.0)) THEN
            GO TO 20
         ELSE
            GO TO 30
         END IF
C
   20    IF ((COORDC.EQ.0).AND.(COOFF.EQ.0)) THEN
            CALL STTPUT
     +      ('switch cursor(s) on - next time we exit...',ISTAT)
            FRAME(1:)  = ' '
            COOFF = 1
            GO TO 10
         ELSE
            CALL DTCLOS(QDSPNO)
            GO TO 50
         END IF
C  
   30    COORDC = COORDC + 1                       !  update coordinate counter
C
C ***    compute the radius of the aperture
C ***    world coordinates of the area center
         XE       = (PXLS1(5)+PXLS2(5))/2.
         YE       = (PXLS1(6)+PXLS2(6))/2.
         RX       =  ABS(PXLS2(5)-PXLS1(5))/2.0
         RY       =  ABS(PXLS2(6)-PXLS1(6))/2.0
         RAYON    =  MIN(RX,RY)
C 
C ***    draw the aperture in the overlay channel
         COORD(1) = (XY1(1)+XY2(1))/2
         COORD(2) = (XY1(2)+XY2(2))/2
         NX       = (XY2(1)-XY1(1))/2
         NY       = (XY2(2)-XY1(2))/2
         COORD(3) = MIN(NX,NY)
         CALL BLDGRA(CERCLE,COORD,TEMP,XFIG,YFIG,2048,NOP)
         CALL IIGPLY(QDSPNO,QOVCH,XFIG,YFIG,NOP,255,1,ISTAT)
C
         IF (BEAM.NE.-1.) THEN
            RAYON = BEAM
         ENDIF
C
         CALL SFLUX(MADRID(WPTR),MADRID(IP),IMF,NAXIS,NPIX,START,STEP,
     +              XE,YE,PIXFLG,RAYON,BGSB,ACAT)
         IF (COORDC.EQ.1) THEN
            CALL STTPUT
     +      ('    Center (w.c.)    Radius (w.c.)  #Pixels'//
     +       '   Flux     Background  Magnitude',ISTAT)
         ENDIF
C
         IF (ACAT(7).LE.-1000.) THEN
            ACAT(1) = XE
            ACAT(2) = YE
            ACAT(3) = 0.0
            ACAT(4) = 0.0
            ACAT(5) = 0.0
            ACAT(6) = 0.0
            ACAT(7) = 0.0
         ENDIF
 
         WRITE(TEXT,902) ACAT(1),ACAT(2),ACAT(3),INT(ACAT(4)),ACAT(5),
     +                   ACAT(6),ACAT(7)
         CALL STTPUT(TEXT,ISTAT) 
         IF (OUTAB(1:1).NE.'?') THEN
            CALL TBRWRR(TIDO,COORDC,7,OCOL,ACAT,ISTAT)
         ENDIF
         GOTO 10
C 
C  here the table input option
C 
      ELSE               
         RAYON = ABS(BEAM)
         CALL FPXWCO(0,IMF,CRDIN,CRDOUT,ISTAT)     ! init world coord conv.
         IF (ISTAT .GT. 0)  THEN
            CALL STETER(37,
     +      'Problem with setup of : world coord conversion')
         ENDIF
         BEAM = 2 * RAYON                ! we use start + 2*radius as endpoint
         IF (PIXFLG.EQ.1) THEN           ! we work with wc's
            IF (STEP(1) .LT.0.0) THEN
               CRDIN(1) = START(1) - BEAM 
            ELSE
               CRDIN(1) = START(1) + BEAM 
            ENDIF
            IF (STEP(2) .LT.0.0) THEN
               CRDIN(2) = START(2) - BEAM 
            ELSE
               CRDIN(2) = START(2) + BEAM 
            ENDIF
            IF (NAXIS .GT. 2) THEN
               CRDIN(3) = START(3)
            ELSE
               CRDIN(3) = 0.D0
            ENDIF
            CALL FPXWCO(-1,IMF,CRDIN,CRDOUT,ISTAT)    ! wc -> fp
            IF (ISTAT .GT. 0) THEN
               CALL STETER(38,
     +         'Problem with conversion: world coord -> frame pixels')
            ENDIF
            NX = CRDOUT(1) + 2          ! + 2 for security
            NY = CRDOUT(2) + 2          ! + 2 for security
            IF ((NX.LT.1) .OR. (NX.GT.NPIX(1)) .OR.
     +          (NY.LT.1) .OR. (NY.GT.NPIX(2))) THEN
               CALL STETER(77,'bad radius...')  
            ENDIF
            COLX   = 'X_COORD'
            COLY   = 'Y_COORD'
         ELSE
            NX = NINT(BEAM) + 3          ! 2* radius + 1 + 2 (for security)
            NY = NINT(BEAM) + 3
            IF ((NX.GT.NPIX(1)).OR.(NY.GT.NPIX(2))) THEN
               CALL STETER(77,'bad radius...')  
            ENDIF
            COLX   = 'X_COORDPIX'
            COLY   = 'Y_COORDPIX'
         ENDIF
CC      write(*,123)nx,ny
CC123   format('we mapped nx,ny = ',2I5)

         CALL STFXMP(NX*NY,D_R4_FORMAT,WPTR,ISTAT)
C ***    read coordinates XE,YE in the input table
C ***    Initialisation of the labels of the columns
         CALL TBTOPN(INTAB,F_I_MODE,TIDI,ISTAT)
C
C *** Find columns X_COORD and Y_COORD
         CALL TBLSER(TIDI,COLX,ICOL(1),ISTAT)
         IF (ICOL(1).LE.0) THEN
            CALL STTPUT('The column label for X is not present',
     +                   ISTAT)
            CALL TBTCLO(TIDI,ISTAT)
            GO TO 50
         END IF
         CALL TBLSER(TIDI,COLY,ICOL(2),ISTAT)
         IF (ICOL(2).LE.0) THEN
            CALL STTPUT('The column label for Y is not present',
     +                   ISTAT)
            CALL TBTCLO(TIDI,ISTAT)
            GO TO 50
         END IF
         NCOL = 2
C
C ***    Find number of rows
         CALL TBIGET(TIDI,N1,NROW,N2,NACOL,NAROW,ISTAT)
C
C ***    Read each row for XE,YE
         I = 0
   40    I = I + 1
         IF (I.GT.NROW) THEN
            CALL TBTCLO(TIDI,ISTAT)
            GO TO 50
         ENDIF
C
         CALL TBSGET(TIDI,I,SELFLG,ISTAT)        !only use selected rows
         IF (.NOT. SELFLG) GOTO 40

         CALL TBRRDR(TIDI,I,2,ICOL,XEYE,TABNUL,ISTAT)
         XE = XEYE(1)
         YE = XEYE(2)
         COORDC = COORDC + 1
C
         CALL SFLUX(MADRID(WPTR),MADRID(IP),IMF,NAXIS,NPIX,START,STEP,
     +              XE,YE,PIXFLG,RAYON,BGSB,ACAT)
         IF (I.EQ.1) THEN
            CALL STTPUT
     +      ('    Center (w.c.)    Radius (w.c.)  #Pixels'//
     +       '   Flux     Background  Magnitude',ISTAT)
         ENDIF
C
         IF (ACAT(7).LE.-1000) THEN
            ACAT(1) = XE
            ACAT(2) = YE
            ACAT(3) = 0.0
            ACAT(4) = 0.0
            ACAT(5) = 0.0
            ACAT(6) = 0.0
            ACAT(7) = 0.0
         ENDIF
C 
         WRITE(TEXT,902) ACAT(1),ACAT(2),ACAT(3),INT(ACAT(4)),ACAT(5),
     +                   ACAT(6),ACAT(7)
         CALL STTPUT(TEXT,ISTAT) 
         IF (OUTAB(1:1).NE.'?') THEN
            CALL TBRWRR(TIDO,COORDC,7,OCOL,ACAT,ISTAT)
         ENDIF
         GOTO 40
      END IF
C
C *** That's it folks...
   50 CONTINUE
C
C *** put the last entry in the keyword OUTPUTR
      CALL STKWRR('OUTPUTR',ACAT,1,7,KUN,ISTAT)
      IF (OUTAB(1:1).NE.'?') CALL TBTCLO(TIDO,ISTAT)
C
      CALL STSEPI
      END


C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C.IDENTIFICATION: SFLUX
C.AUTHOR:         Ch. Ounnas                  ESO - Garching
C.LANGUAGE:       F77+ESOext
C.KEYWORDS:       flux-magnitude
C.PURPOSE:        compute flux inside an aperture
C.VERSION:        880919 RHW ESO-FORTRAN Conversion
C.VERSION:        910115 RHW IMPLICIT NONE added
C ----------------------------------------------------------------------
      SUBROUTINE SFLUX(RMES,FMES,IMP,NAXIS,NPIX,START,STEP,XE,YE,
     +                 PIXFLG,RAYON,BGSB,TAB)
C
      IMPLICIT   NONE
C
      REAL       RMES(1), FMES(1)
      INTEGER    IMP,NAXIS,NPIX(2)
      DOUBLE PRECISION START(*),STEP(*)
      REAL       XE,YE
      INTEGER    PIXFLG
      REAL       RAYON,BGSB,TAB(7)
C 
      INTEGER    IMIN, IMAX
      INTEGER    LMIN, LMAX
      INTEGER    IST
      INTEGER    LM, IL, IM
      INTEGER    NPL
      INTEGER    NXP,NYP
C     
      REAL       DIF, DELTAS
      REAL       DX, DY
      REAL       PIX,RADIUS
      REAL       PS2, POIDS
      REAL       R2
      REAL       RES
      REAL       XP, YP
      REAL       XC, YC
      REAL       XSTA, YSTA
      REAL       XMIN, XMAX
      REAL       YMIN, YMAX
      REAL       ZERO
      REAL       VMAG
C 
      DOUBLE PRECISION CRDIN(3),CRDOUT(3)
C 
      CHARACTER  ERSTRG*8,TEXT*60
C
      DX     = STEP(1)
      DY     = STEP(2)
      XSTA   = START(1)
      YSTA   = START(2)
C
      DELTAS = ABS(DY*DX)
C 
      ZERO   = 0.
      XMIN   = XE - RAYON
      XMAX   = XE + RAYON
      YMIN   = YE - RAYON
      YMAX   = YE + RAYON
      TAB(7) = 0.0
C
      IF (PIXFLG.EQ.1) THEN
         RADIUS = RAYON                  ! radius in w.c.
         CRDIN(1) = XMIN 
         CRDIN(2) = YMIN 
         IF (NAXIS.GT.2) CRDIN(3) = START(3)
         CALL FPXWCO(-1,IMP,CRDIN,CRDOUT,IST)    ! world coord -> frame pixel
         IF (IST .EQ. 0) THEN
            IMIN = NINT(CRDOUT(1))
            LMIN = NINT(CRDOUT(2))
            IF ((IMIN.LT.1) .OR. (IMIN.GT.NPIX(1)) .OR.
     +          (LMIN.LT.1) .OR. (LMIN.GT.NPIX(2))) THEN
               ERSTRG = '(lo x,y) '
               GOTO 40
            ENDIF
         ELSE
            ERSTRG = '(lo x,y) '
            GOTO 40
         ENDIF
C
         CRDIN(1) = XMAX 
         CRDIN(2) = YMAX 
         CALL FPXWCO(-1,IMP,CRDIN,CRDOUT,IST)    ! world coord -> frame pixel
         IF (IST .EQ. 0) THEN
            IMAX = NINT(CRDOUT(1))
            LMAX = NINT(CRDOUT(2))
            IF ((IMAX.LT.1) .OR. (IMAX.GT.NPIX(1)) .OR.
     +          (LMAX.LT.1) .OR. (LMAX.GT.NPIX(2))) THEN
               ERSTRG = '(hi x,y) '
               GOTO 40
            ENDIF
         ELSE
            ERSTRG = '(hi x,y) '
            GOTO 40
         ENDIF
C 
         IF (DX.LE.0.) THEN
            IM = IMIN
            IMIN = IMAX
            IMAX = IM
         END IF
C
         IF (DY.LE.0.) THEN
            IM = LMIN
            LMIN = LMAX
            LMAX = IM
         END IF
C
      ELSE                        ! here we work with frame pixels
         RADIUS = RAYON * ABS(DX)      ! radius now in w.c.
         CRDIN(1) = XE 
         CRDIN(2) = YE 
         IF (NAXIS.GT.2) CRDIN(3) = 1.D0
         CALL FPXWCO(1,IMP,CRDIN,CRDOUT,IST)    ! frame pixel -> world coord 
         IF (IST .EQ. 0) THEN
            XE = SNGL(CRDOUT(1))
            YE = SNGL(CRDOUT(2))
         ELSE
            ERSTRG = '(c. x,y) '
            GOTO 40
         ENDIF
         IMIN = NINT(XMIN)
         LMIN = NINT(YMIN)
         IF ((IMIN.LT.1) .OR. (IMIN.GT.NPIX(1)) .OR.
     +       (LMIN.LT.1) .OR. (LMIN.GT.NPIX(2))) THEN
            ERSTRG = '(lo x,y) '
            GOTO 40
         ENDIF
         IMAX = NINT(XMAX)
         LMAX = NINT(YMAX)
         IF ((IMAX.LT.1) .OR. (IMAX.GT.NPIX(1)) .OR.
     +       (LMAX.LT.1) .OR. (LMAX.GT.NPIX(2))) THEN
            ERSTRG = '(hi x,y) '
            GOTO 40
         ENDIF
      ENDIF
C
      IM = IMAX - IMIN + 1
      LM = LMAX - LMIN + 1
      PIX = 0.
      RES = 0.
      PS2 = DX/2.
C 
      DO 20, IL = 1,LM!loop over y-lines
         CALL LIRE(IL+LMIN-1,NPIX(1),IMIN,IMAX,1,FMES,RMES)
         NYP = 1 + LMIN + IL -2          ! YP = YSTA + FLOAT(LMIN+IL-2)*DY
         IF (DY.LT.0.) THEN
            CALL LIRE(LMAX-IL+1,NPIX(1),IMIN,IMAX,1,FMES,RMES)
            NYP = 1 + LMAX - IL          ! YP = YSTA + FLOAT(LMAX-IL)*DY
         END IF
C
         DO 10, NPL = 1,IM               !loop over x-pixels
            NXP = 1 + IMIN + NPL -2      ! XP = XSTA + FLOAT(IMIN+NPL-2)*DX
            IF (DX.LT.0.) THEN
               NXP = 1 + IMAX - NPL      ! XP = XSTA + FLOAT(IMAX-NPL)*DX
            END IF
C
            CRDIN(1) = DBLE(NXP)
            CRDIN(2) = DBLE(NYP)
            CALL FPXWCO(1,IMP,CRDIN,CRDOUT,IST)    ! frame pix -> world coord
            IF (IST .EQ. 0) THEN
               XP = SNGL(CRDOUT(1))
               YP = SNGL(CRDOUT(2))
            ELSE
               CALL STTPUT('*** INFO: Problems in fp -> wc conversion',
     +                     IST)
               GO TO 44
            ENDIF
C
            YC = (YE-YP)**2
            XC = (XE-XP)**2
            R2 = SQRT(YC+XC)
            DIF = RADIUS - R2
C
            IF (DIF.GE.0.) THEN
               IF (DIF.GT.PS2) THEN
                  POIDS = 1.
               ELSE
                  POIDS = 0.5* (1.+DIF/PS2)
               END IF
            ELSE
               IF (ABS(DIF).GT.PS2) THEN
                  POIDS = 0.
               ELSE
                  POIDS = 0.5* (1+DIF/PS2)
               END IF
            END IF
C
            RES = RES + POIDS*RMES(NPL)
            PIX = PIX + POIDS
   10    CONTINUE
   20 CONTINUE
C
      VMAG = RES - PIX*BGSB
      IF (VMAG.GT.0) THEN
         VMAG = -2.5*ALOG10(VMAG)
      ELSE
         VMAG = -9999.99
      END IF
C
      TAB(1) = XE
      TAB(2) = YE
      TAB(3) = RADIUS
      TAB(4) = AINT(PIX)
      TAB(5) = RES
      TAB(6) = BGSB
      TAB(7) = VMAG
      RETURN
C 
40    TEXT = '*** INFO: Aperture area outside frame boundaries '
      CALL STTPUT(TEXT // ERSTRG,IST)
44    TAB(7) = -9999.99
      RETURN
C 
      END


      SUBROUTINE LIRE(NL,NPL,NPL1,NPL2,NPL3,FMES,RMES)
C+++
C.PURPOSE:  Write part of a frame into an array
C.AUTHOR:   ???
C.VERSION:  ?????? ??? created
C.VERSION:  890117 RHW documented
C.COMMENTS: none
C---
      IMPLICIT   NONE
      INTEGER    NL
      INTEGER    NPL
      INTEGER    NPL1
      INTEGER    NPL2
      INTEGER    NPL3
      REAL       FMES(1)
      REAL       RMES(1)                                                

      INTEGER    NPD, NPF, K, I 
C

C ***
      NPD = NPL* (NL-1) + NPL1 
      NPF = NPD + NPL2 - NPL1 
      K   = 0
C 
      DO 10 I = NPD,NPF,NPL3 
         K       = K + 1   
         RMES(K) = FMES(I)
   10 CONTINUE
C
      RETURN
      END
